# RTI.R - N Maizlish - January 20, 2020
# This program reads CRSS 2016 csv data files (ACCIDENT< PERSON< VEHICLE)
# and aggregates severe non-fatal injuries for striking vehicle-victim vehicle  
# matrix by mode ftp://ftp.nhtsa.dot.gov/CRSS/2016/) 
# For alternative years, rename ACCIDENT, PERSON, VEHICLE files with year
# FILE + YEAR (e.g. 2018): ACCIDENT2018.csv, PERSON2018.csv, VEHICLE2018.csv

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Initialize Progrram
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# turn off scientific notation in data frames
options(scipen=999)
crss <- "C:/Users/Noneof/Documents/HealthLinks/ITHIM_MPO/USA/RTI/CRSS"
setwd(crss)
library(survey)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read CRSS Person file and Subset for severe injuries (INJ=SEV= 3) 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

person <- read.csv("PERSON2016.csv", header = TRUE, stringsAsFactors = F)  
person$victim_id <- paste0(person$CASENUM,person$VEH_NO, person$PER_NO)
# person$party_id  <- paste0(person$CASENUM,person$VEH_NO)
person           <- person[person$INJ_SEV==3,c("victim_id", "CASENUM","VEH_NO",
                          "PER_NO","PER_TYP","INJ_SEV", "VE_FORMS")]

# Create victim mode for pedestrians and cyclists (9 = missing)
person$VMODE <- NA
person[person$VEH_NO==0 & person$PER_TYP==5, "VMODE"] <- 1
person[person$VEH_NO==0 & person$PER_TYP==6, "VMODE"] <- 2

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read CRSS Vehicle file to get parties to the severe injury collision
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

vehicles <- read.csv("VEHICLE2016.csv", header = TRUE, stringsAsFactors = F)
vehcodes <- read.csv("BodyTypeCodes.csv", header = TRUE, stringsAsFactors = F)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with Vehicle codes list to get ITHIM mode for motorized vehicles
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
vehicles <- merge(vehicles, vehcodes, by="BODY_TYP", all.x = TRUE) 

vehicles <- vehicles[,c("CASENUM", "VEH_NO", "VehType","VehMode","VTRAFWAY",
  "VSPD_LIM", "WEIGHT")]

# Deduplicate taking lowest (highest speed) for roadway type

vehicles <- vehicles[order(vehicles$CASENUM,vehicles$VSPD_LIM),]
vehicle2 <- vehicles[!(duplicated(vehicles$CASENUM)),]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create Roadway type (highway, arterial, and local) from speed (VSPD_LIM) 
# and Tafficway (TRAFWAY) in vehicle2 file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Speed and Trafficway
vehicle2$Roadway <- NA
vehicle2[vehicle2$VSPD_LIM <= 30  & vehicle2$VTRAFWAY %in% c(0,1,4), "Roadway" ] <- "Local"
vehicle2[vehicle2$VSPD_LIM >= 35  & vehicle2$VSPD_LIM <= 55 & vehicle2$VTRAFWAY %in% c(2,5), "Roadway" ] <- "Arterial"
vehicle2[vehicle2$VSPD_LIM >= 60  & vehicle2$VTRAFWAY %in% c(3,6), "Roadway" ] <- "Highway"

# Fine tune missing data
vehicle2[is.na(vehicle2$Roadway)  & vehicle2$VTRAFWAY == 2 & vehicle2$VSPD_LIM >= 70, "Roadway" ] <- "Highway"
vehicle2[is.na(vehicle2$Roadway)  & vehicle2$VTRAFWAY == 3 & vehicle2$VSPD_LIM >= 50, "Roadway" ] <- "Highway"
vehicle2[is.na(vehicle2$Roadway)  & vehicle2$VTRAFWAY == 2 & vehicle2$VSPD_LIM <= 60, "Roadway" ] <- "Arterial"
vehicle2[is.na(vehicle2$Roadway)  & vehicle2$VTRAFWAY == 1 & vehicle2$VSPD_LIM <= 30, "Roadway" ] <- "Local"
vehicle2[is.na(vehicle2$Roadway), "Roadway" ] <- "Arterial"

# Missing both VSPD_LIM and VTRAFWAY
vehicle2[vehicle2$VSPD_LIM >= 98 & vehicle2$VTRAFWAY >= 8, "Roadway" ] <- NA

# Keep Casenum, Roadway type and WEIGHT 
vehicle2 <- vehicle2[,c("CASENUM", "Roadway", "WEIGHT")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Assign victim mode for all severe injuries (non-motorized + motorized)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Person file with all victims, including non-vehicle victims
person <- merge(person, vehicles,  by = c("CASENUM","VEH_NO"), all.x = TRUE)

# Assign mode type for vehicular serious injuries
person$VMODE <- ifelse(is.na(person$VMODE), person$VehMode, person$VMODE)
person <- person[order(person$victim_id),c("victim_id", "CASENUM","PER_NO",
                "VE_FORMS", "VMODE")]

# Assign missing victim mode
person[is.na(person$VMODE),"VMODE"] <- 7

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with Vehicle file to pick up all vehicles involved in crash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

person_vehicles <- merge(person, vehicles, by = c("CASENUM"), all.x = TRUE)
person_vehicles <- person_vehicles[,c("victim_id","VE_FORMS","VMODE", "VEH_NO",
                  "VehMode")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create wide file with all parties to a collision on the same row
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Wide file with victim and all parties
victims <- reshape(person_vehicles,
                   timevar = "VEH_NO",
                   idvar = c("victim_id", "VMODE","VE_FORMS"),
                   direction = "wide")

# Count of rows with missing data for striking vehicle
apply(victims[,c(4:ncol(victims))],2,function(x) sum(!is.na(x)))

# VehMode.1 VehMode.2 VehMode.4 VehMode.3 VehMode.5 VehMode.6 VehMode.7 
# 4945      2975       161       552        49        16         7 


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Striking Vehicle in collision with 2 or more parties, is mode with
# the highest numerical value (mass) that isn't the victim mode, unless
# other parties are the same mode as victim mode
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Striking Vehicle
# Mass 
# Precedence Code
# ~~~~~~~~~~~~~~~~

# Pedestrian  1
# Bicycle     2
# Motorcycle  3
# Car         4
# Bus         5
# Truck       6
# NOV         7
# Missing     8
# ~~~~~~~~~~~~~~~~

# Striking Vehicle with Largest mass
victims$max    <- apply(victims[,c(4:10)], MARGIN=1, FUN=max, na.rm=TRUE)
victims$sumveh <- apply(victims[,c(4:10)], MARGIN=1, function(x) sum(!is.na(x)))
victims$SMODE  <- ifelse(victims$max > victims$VMODE, victims$max, victims$VMODE)

# Striking mode NOV for non-motorized and missing victims
victims$SMODE <- ifelse(victims$VMODE <= 2 & victims$sumveh==0, 7, victims$SMODE)

# Striking Mode NOV for motorized modes
victims$SMODE <- ifelse(victims$VMODE >= 3 & victims$sumveh==1, 7, victims$SMODE)

# Striking Mode of Bus victims of car collisions
victims$SMODE <- ifelse(victims$VMODE == 5 & victims$sumveh == 2 &
                  ((victims$VehMode.1 == 5 & victims$VehMode.2 == 4) | 
                  ( victims$VehMode.1 == 4 & victims$VehMode.2== 5)), 4,
                  victims$SMODE)

# Striking Mode of Truck victims of car collisions
victims$SMODE <- ifelse(victims$VMODE == 6 & victims$sumveh == 2 &
                          ((victims$VehMode.1 == 6 & victims$VehMode.2 == 4) | 
                          ( victims$VehMode.1 == 4 & victims$VehMode.2 == 6)), 4, 
                        victims$SMODE)

# Striking Mode of Truck victims of bus collisions
victims$SMODE <- ifelse(victims$VMODE == 6 & victims$sumveh==2 &
                          ((victims$VehMode.1 == 6 & victims$VehMode.2 == 5) | 
                          (victims$VehMode.1 == 5 & victims$VehMode.2== 6)), 5, 
                        victims$SMODE)

# Assign Striking mode Missing (8) for missing Victim mode (7)
victims$SMODE <- ifelse(victims$VMODE == 7 & victims$sumveh==0, 8, victims$SMODE)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Pick up roadway type from Collision file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

victims$CASENUM <- floor(as.numeric(victims$victim_id)/100)
victims <- merge(victims, vehicle2, by="CASENUM", all.x = TRUE)
victims <- victims[,c("victim_id","CASENUM","VMODE","SMODE","Roadway","WEIGHT")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Unweighted Victim-Striking Vehicle Matrix by Roadway
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Totals including missing roadway type
tab_ttl <- as.data.frame.matrix(addmargins(table(victims$VMODE,victims$SMODE)))

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Pick up PSU and Strata from Accident file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

accident <- read.csv("ACCIDENT2016.csv", header = TRUE, stringsAsFactors = F)  
accident <- accident[,c("CASENUM","PSU_VAR","PSUSTRAT")]
victims  <- merge(victims,accident, by="CASENUM", all.x=TRUE) 

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Weighted estimate of ictim-Striking Vehicle Matrix by Roadway
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

library(survey)
survey <- svydesign(id=~PSU_VAR, strata=~PSUSTRAT, weights=~WEIGHT,  data=victims)
wtd_total <- as.data.frame.matrix(addmargins(svytable(~VMODE+SMODE, survey)))

temp <- svytable(~VMODE+SMODE, survey)

# Missing Roadway = wtd_total[7,6]
# wtd_total[7,6]/wtd_total[8,7]

#               ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~Striking  Mode ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Victim Mode   Walk(1) Bike(2) Motorcycle(3) Car(4) Bus(5) Truck(6) NOV(7) Missing(8)  Sum(9)
#       Walk(1)
#       Bike(2)
# Motorcycle(3)
#        Car(4)
#        Bus(5)
#      Truck(6)
#    Missing(7)
#       Sum (8)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create function for imputation stratified by roadway type
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
impute.function <- function(fRoadway) {

roadtype <- fRoadway

# Subset victims by roadway type (not used for SE)
victtmp <- victims[!is.na(victims$Roadway) & victims$Roadway==roadtype,]

# Set survey design (used just for weights)
svyrti  <- svydesign(id=~PSU_VAR, strata=~PSUSTRAT, weights=~WEIGHT, data=victtmp)

# Compute weighted victim-strikign vehicle matrix
tabtemp  <- as.data.frame.matrix(addmargins(svytable(~VMODE+SMODE, svyrti)))

# Bus injuries are 0 for local and highways, so filler row is need so
# imputation will have correct number of rows

filler <- c(0,0,0,0,0,0,0)
if (roadtype == "Highway" | roadtype == "Local") {
  tabtemp <- rbind(tabtemp[1:4,],filler,tabtemp[5:7,])
}

# Add Walk and Bike Striking modes
tabtemp[,c("1","2")] <- 0
tabtemp <- tabtemp[,c("1","2","3","4","5","6","7","8","Sum")]
rti <-  tabtemp

# Impute missing values
newrti <- as.data.frame(matrix(0, ncol = 7, nrow = 6))

# Loop over the ith row and jth column
for(i in 1:6) {
  for(j in 1:7) {
newrti[i,j] = (rti[i,j] +((rti[i,j]/(rti[i,9]-rti[i,8])*rti[i,8])+(rti[i,j]/(rti[8,j]-rti[7,j])*rti[7,j])))*(rti[8,9]/(rti[8,9]-rti[7,8]))
  } # End jth column
}  # End ith row

# Replace all division by 0 with 0s
newrti[is.na(newrti)] <- 0

# Adjust by missing roadway type 
newrti <- newrti*(1/(1 - wtd_total[7,6]/wtd_total[8,7]))

# Add column names
colnames(newrti)[1:7] <- c("walk", "bike", "motorcycle", "car","bus", "truck", "NOV")

newrti$Region <- "United States"
newrti$Severity <- "Serious"
newrti$Roadway  <- roadtype
newrti$VictimMode <- NA
newrti[1,"VictimMode"] <- "1Walk"
newrti[2,"VictimMode"] <- "2Bike"
newrti[3,"VictimMode"] <- "6Motorcycle"
newrti[4,"VictimMode"] <- "4Car"
newrti[5,"VictimMode"] <- "3Bus"
newrti[6,"VictimMode"] <- "5Truck"

# Reorder striking Mode columns
newrti <- newrti[,c("Region", "Severity", "Roadway", "VictimMode", "walk", 
                    "bike", "bus", "car", "truck", "motorcycle", "NOV")]

# Reorder Victim Mode rows
newrti <- newrti[order(newrti$VictimMode),]

# strip off ordering character
newrti$VictimMode <- substr(newrti$VictimMode,2,11)

return(newrti)
}  

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Impute Missing Data for Each Roadway type
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
hwy_tab  <- impute.function("Highway")
art_tab  <- impute.function("Arterial")
loc_tab  <- impute.function("Local")

# Check sum
sum(hwy_tab[,5:11]) + sum(art_tab[,5:11]) + sum(loc_tab[,5:11])

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Consolidate in a single file 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

serious_rti <- rbind(loc_tab,art_tab,hwy_tab)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out resuls
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(serious_rti, paste0("rti_baseline_serious",format(Sys.time(),
           "%Y-%m-%d"),".csv"), row.names=FALSE)

