# CVnonTravelMETS.R - N Maizlish - January 9, 2020
# This program reads SAS XPT files for NHANES, 2015-2016 and calculates
# age-sex nontravel METS (leisure, work) at quintiles of travel related METS
# (walking and cycling). 
# https://wwwn.cdc.gov/nchs/nhanes/search/datapage.aspx?Component=Questionnaire&CycleBeginYear=2015
# The program also calculates the standard deviation of physical activity time 
# of active travel incorporating teh sample design in estimates
# The anaysis requires the packages: surveyHmisc

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Initialize Progrram
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# turn off scientific notation in data frames
options(scipen=999)
hanes <- "C:/Users/Noneof/Documents/HealthLinks/ITHIM_MPO/USA/CalibrationData/NHANES"
setwd(hanes)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read NHANES demographics file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
library(Hmisc)
library(data.table)

person <- sasxport.get("DEMO_I.XPT")
person <- person[,c("seqn","riagendr","ridageyr","wtint2yr","sdmvpsu","sdmvstra")]

# seqn - smaple person ID number (for linkage)
# riagendr - Gender of the participant (1 = M, 2 = F)
# ridageyr - age at screening, 80+ years top coded as 80
# wtint2yr - sample weight for 2 year cycle
# sdmvpsu  - primary sample unit (masked)
# sdmvstra - sample strata (masked)

age_groups <- function(x) { 
  if    (is.na(x)) y <- "NA"
  else if (x < 5)              y <- 0
  else if (x >= 5  & x <= 14)  y <- 5
  else if (x >= 15 & x <= 29)  y <- 15
  else if (x >= 30 & x <= 44)  y <- 30
  else if (x >= 45 & x <= 59)  y <- 45
  else if (x >= 60 & x <= 69)  y <- 60
  else if (x >= 70 & x <= 79)  y <- 70
  else if (x >= 80)            y <- 80
  else y <- -99
  return(y)
}

# ITHIM Age Sex Groups
person$Age <- sapply(person$ridageyr,age_groups)
names(person)[names(person)=="riagendr"] <- "Sex"
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read NHANES physical activity file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

paq <- sasxport.get("PAQ_I.XPT")
paq <- paq[,c("seqn", "paq605", "paq610", "pad615", "paq620", "paq625", "pad630",
       "paq635", "paq640", "pad645", "paq650", "paq655", "pad660", "paq665", 
       "paq670", "pad675", "paq706")]
            
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Ages 12 and older
# PA at Work
# PAQ605 - Vigorous work activity (1 =Yes, 2=No, 7=refused, 9=DK, blank = missing)
# PAQ610 - Days/week vigorous work (1-7, 77=Refused, 99 = DK, b = missing)
# PAD615 - Minutes/d vigorous-intensity work (1-840, 7777=Refused, 9999=DK, b=missing)
# PAQ620 - Moderate work activity (1 =Yes, 2=No, 7=refused, 9=DK, blank = missing)
# PAQ625 - Number of days per week moderate work (1-7, 77=Refused, 99 = DK, b = missing)
# PAD630 - Minutes moderate -intensity work (1-990, 7777=Refused, 9999=DK, b=missing)

# PA for Transportation
# PAQ635 - In a typical week walk or use a bicycle for at least 10 minutes continuously 
#          to get to and from places? (1 =Yes, 2=No, 7=refused, 9=DK, blank = missing)
# PAQ640 - Days/week walk or bicycle ((1-7, 77=Refused, 99 = DK, b = missing)
# PAD645 - Minutes/day walk/bicycle for transportation (1-1200, 7777=Refused, 9999=DK, b=missing)

# PA for Leisure
# PAQ650 - Vigorous recreational activity (1 =Yes, 2=No, 7=refused, 9=DK, blank = missing)
# PAQ655 - Days/week vigorous rec activity (1-7, 77=Refused, 99 = DK, b = missing)
# PAD660 - Minutes/day vigorous-intensity rec activity (1-480, 7777=Refused, 9999=DK, b=missing)
# PAQ665 - Moderate recreational activity (1 =Yes, 2=No, 7=refused, 9=DK, blank = missing)
# PAQ670 - Days/week moderate rec activity (1-7, 77=Refused, 99 = DK, b = missing)
# PAD675 - Minutes/day moderate-intensity rec activity (1-600, 7777=Refused, 9999=DK, b=missing)

# Ages 2 to 15 years of age
# PAQ706 - Days/w with >=60 minutes/day of moderate and vigorous PA (0-7, 77=Refused, 
# 99 = DK, b = missing)

# Label                                 METs
# ------------------------------------------
# Vigorous work-related activity           8
# Moderate work-related activity           4
# Walking or bicycling for transportation  4
# Vigorous leisure-time physical activity  8
# Moderate leisure-time physical activity  4

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Recode for 0 and missing; Refused and DK set to 0
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Work: min/week Vigorous PA
paq$dpw_vig_work <- ifelse(paq$paq610 > 7   | is.na(paq$paq610), 0, paq$paq610)
paq$mpd_vig_work <- ifelse(paq$pad615 > 840 | is.na(paq$pad615), 0, paq$pad615)
paq$mpw_vig_work <- paq$dpw_vig_work*paq$mpd_vig_work

# Work: min/week moderate moderate PA
paq$dpw_mod_work <- ifelse(paq$paq625 > 7    | is.na(paq$paq625), 0, paq$paq625)
paq$mpd_mod_work <- ifelse(paq$pad630 > 990  | is.na(paq$pad630), 0, paq$pad630)
paq$mpw_mod_work <- paq$dpw_mod_work*paq$mpd_mod_work

# transport: min/week walk and bike with 2 hr/d * 5 d/w = 600 max
paq$dpw_at <- ifelse(paq$paq640 > 7    | is.na(paq$paq640), 0, paq$paq640)
paq$mpd_at <- ifelse(paq$pad645 > 1200 | is.na(paq$pad645), 0, paq$pad645)
paq$mpw_at <- paq$dpw_at*paq$mpd_at
paq$mpw_at <- ifelse(paq$mpw_at > 600, 600, paq$mpw_at) 

summary(paq$mpw_at)
histogram(paq$mpw_at, breaks = 75 )

# Leisure: min/week Vigorous PA
paq$dpw_vig_rec <- ifelse(paq$paq655 > 7   | is.na(paq$paq655), 0, paq$paq655)
paq$mpd_vig_rec <- ifelse(paq$pad660 > 480 | is.na(paq$pad660), 0, paq$pad660)
paq$mpw_vig_rec <- paq$dpw_vig_rec*paq$mpd_vig_rec

# Leisure: min/week moderate moderate PA
paq$dpw_mod_rec <- ifelse(paq$paq670 > 7   | is.na(paq$paq670), 0, paq$paq670)
paq$mpd_mod_rec <- ifelse(paq$pad675 > 600 | is.na(paq$pad675), 0, paq$pad675)
paq$mpw_mod_rec <- paq$dpw_mod_rec*paq$mpd_mod_rec

# Calculate total PA min per week for quality control
paq$mpw_pa_ttl <- paq$mpw_vig_work + paq$mpw_mod_work + paq$mpw_at +
  paq$mpw_vig_rec + paq$mpw_mod_rec

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Adjust total PA downward for all categories by proportion of overage 
# Max = 2520 min/wk (i.e. 6h/d*7d/w)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# paq_qa <- paq[paq$mpw_pa_ttl > 2520, c("mpw_vig_work", "mpw_mod_work", "mpw_at",      
#             "mpw_vig_rec", "mpw_mod_rec", "mpw_pa_ttl")]

paq$mpw_pa_adj   <- paq$mpw_pa_ttl/2520

paq$mpw_vig_work <- ifelse(paq$mpw_pa_adj > 1, paq$mpw_vig_work/paq$mpw_pa_adj, paq$mpw_vig_work)
paq$mpw_mod_work <- ifelse(paq$mpw_pa_adj > 1, paq$mpw_mod_work/paq$mpw_pa_adj, paq$mpw_mod_work)
paq$mpw_vig_rec  <- ifelse(paq$mpw_pa_adj > 1, paq$mpw_vig_rec/paq$mpw_pa_adj, paq$mpw_vig_rec)
paq$mpw_mod_rec  <- ifelse(paq$mpw_pa_adj > 1, paq$mpw_mod_rec/paq$mpw_pa_adj, paq$mpw_mod_rec)

# Re-Calculate total PA min per week for quality control
paq$mpw_pa_ttl <- paq$mpw_vig_work + paq$mpw_mod_work + paq$mpw_at +
  paq$mpw_vig_rec + paq$mpw_mod_rec

# Summary and histogram
summary(paq$mpw_pa_ttl)
histogram(paq$mpw_pa_ttl, breaks = 75 )

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Calculate Non-Travel METS (work + leisure)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# paq$ntMEThr_wk <- (8*paq$mpw_vig_work + 4*paq$mpw_mod_work + 8*paq$mpw_vig_rec 
#                   + 4*paq$mpw_mod_rec)/60

# geometric mean calculation adds 0.1 
paq$ntMEThr_wk <- (8*paq$mpw_vig_work + 4*paq$mpw_mod_work + 8*paq$mpw_vig_rec 
                   + 4*paq$mpw_mod_rec)/60 + 0.1
paq$ntMEThr_wk <- log(paq$ntMEThr_wk)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Calculate Travel METS (walk + bike) from parsing total by NHTS2017
# Walk and bike proportions of total AT
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

setwd("C:/Users/Noneof/Documents/HealthLinks/ITHIM_MPO/USA/CalibrationData/NHTS2017")

at_walkpct <- read.csv("Walk_pct_AT2020-01-13.csv", header = TRUE, stringsAsFactors = F) 

paq <- merge(paq,person[,c("seqn","Sex", "Age")], by="seqn", all.x = TRUE)
paq <- merge(paq,at_walkpct, by = c("Sex","Age"), all.x = TRUE)

#                   4 METS x Walking             + 6 METS for Cycling
paq$travMEThr_wk <- 4*paq$mpw_at*paq$walkpct/60 + 6*paq$mpw_at*(1-paq$walkpct)/60
# histogram(paq$travMEThr_wk, breaks = 75 )

# Create random numbers from 0 to 0.5 for METhrs_wk for 0 values
set.seed(42)
paq$rndno <- runif(nrow(paq))/2

# Create pseudo METhrs for 0 values of travel related METhrs_wk
paq$travMEThr_wk2 <- ifelse(paq$travMEThr_wk == 0, paq$rndno, paq$travMEThr_wk)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# create quintiles of travel related METS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
paq$Q <- as.integer(cut(paq$travMEThr_wk2, quantile(paq$travMEThr_wk2,probs=0:5/5), include.lowest=TRUE))
paq <- paq[order(paq$travMEThr_wk2), ]

# write.csv(paq,paste0("paq", format(Sys.time(),"%Y-%m-%d"),".csv"), row.names = F)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Calculate geometric mean non-travel METhrs-wk for each quintile of travel-
# related METS in age-sex strata
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

paq <- merge(paq[-c(1,2)], person, by = "seqn", all.x = T)

# Unweighted
# ntravMET <- aggregate(paq$ntMEThr_wk~Q+Sex+Age, data=paq, FUN=mean)
# names(ntravMET)[names(ntravMET)=="paq$ntMEThr_wk"] <- "q"

# Weighted
ntravMET <- setDT(paq)[, weighted.mean(ntMEThr_wk, w = wtint2yr), by = c("Q", "Sex", "Age")]
names(ntravMET)[names(ntravMET)=="V1"] <- "q"

# Backtransform mean
ntravMET$q <- exp(ntravMET$q)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reshape file for format
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ntravMET_wide <- reshape(ntravMET,
             timevar = "Q",
             idvar = c("Sex", "Age"),
             direction = "wide")

ntravMET_wide$Region <- "United States"
names(ntravMET_wide)[3:7] <- paste0("q",1:5)
ntravMET_wide <- ntravMET_wide[order(ntravMET_wide$Sex,ntravMET_wide$Age),
                  c("Region", "Sex","Age","q1", "q2","q3","q4","q5")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
setwd(hanes)
write.csv(ntravMET_wide, paste0("nonTravelMETS", format(Sys.time(),"%Y-%m-%d"),".csv"), row.names = F)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Standard Deviation (CV) of the mean of active travel time 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# write test file for Stata Analysis
# at_min_wk <- paq[,c("wtint2yr","sdmvstra","mpw_at")]
# write.csv(at_min_wk, paste0("nhanes_at", format(Sys.time(),"%Y-%m-%d"),".csv"), row.names = F)
# CV from stata analysis  = 200.1571/50.1789 = 3.9888698

# unweighted CV
unwtCV <- sd(paq$mpw_at)/mean(paq$mpw_at)
unwtCV

# set up output file 
cv <- data.frame(Region = rep("United States", 1), 
                 CV=rep(NA, 1), stringsAsFactors=FALSE)  

library(survey)
survey <- svydesign(id=~sdmvpsu, strata=~sdmvstra, weights=~wtint2yr, nest=TRUE, data=paq)
svymean(~mpw_at, survey)
svyvar (~mpw_at, survey)
sqrt(svyvar(~mpw_at, survey))
cv_at <- sqrt(svyvar(~mpw_at, survey))/svymean(~mpw_at, survey)
cv_at
cv[1,2] <- sqrt(svyvar(~mpw_at, survey))/svymean(~mpw_at, survey)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out CV file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
setwd(hanes)
write.csv(cv, paste0("bike_walk_cv", format(Sys.time(),"%Y-%m-%d"),".csv"), row.names = F)





