# NHTS2017DistTimeMode.R - N Maizlish - 1/11/2020
# Reads NHTS person and trip files and # calculates percapita mean daily 
# distances traveled by mode

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Initialize Progrram
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# set working directory
setwd ('C:/SASUniversityEdition/myfolders/ITHIM/TravelSurveys/NHTS/NHTS2017')

# Get todays date to assign version
today <- date()

# turn off scientific notation
options(scipen=999)

library(data.table)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read Person file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

person <- read.csv(file= "perpub.csv", head=TRUE, sep=",", na.strings=c("NA"))

# Keep variables of interest
person <- person[,c("HHSTATE", "HOUSEID", "PERSONID", "R_AGE_IMP", "R_SEX_IMP", 
                    "WTPERFIN")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create Person ID
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
person$person_id <- paste0(person$HOUSEID,person$PERSONID)

# Create ITHIM age categories
age_groups <- function(x) { 
  if    (is.na(x)) y <- "NA"
  else if (x < 5)              y <- 0
  else if (x >= 5  & x <= 14)  y <- 5
  else if (x >= 15 & x <= 29)  y <- 15
  else if (x >= 30 & x <= 44)  y <- 30
  else if (x >= 45 & x <= 59)  y <- 45
  else if (x >= 60 & x <= 69)  y <- 60
  else if (x >= 70 & x <= 79)  y <- 70
  else if (x >= 80)            y <- 80
  else y <- -99
  return(y)
}

# ITHIM Age Sex Groups
person$Age <- sapply(person$R_AGE_IMP,age_groups)
names(person)[names(person)=="R_SEX_IMP"] <- "Sex"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read Trip file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip <- read.csv(file= "trippub.csv", head=TRUE, sep=",", na.strings=c("NA"))

# TRVLCMIN - duration of trip in minutes
# TRPTRANS - Trip Mode, derived
# TRPMILES - Trip distance in miles, derived from route geometry returned 
# by Google Maps API, or from reported loop-trip distance

# keep variables of interest
trip <-trip[,c("HOUSEID", "PERSONID", "TDTRPNUM","TDCASEID", "TRPMILES", 
               "TRVLCMIN", "TRPTRANS", "DRVR_FLG", "PSGR_FLG", "WTTRDFIN")]

# Set trip miles and trip time to 0 if unknown
trip$TRPMILES <- ifelse(trip$TRPMILES<0, 0, trip$TRPMILES)
trip$TRVLCMIN <- ifelse(trip$TRVLCMIN<0, 0, trip$TRVLCMIN)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create person_id for linkage
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip$person_id <- paste0(trip$HOUSEID,trip$PERSONID)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge trip file with ITHIM mode look-up table 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

mode <- read.csv(file= "lookup_ithim_mode.csv", head=TRUE, sep=",", 
                 stringsAsFactors= F, na.strings=c("NA"))
trip <- merge(trip, mode, by="TRPTRANS", all.x=TRUE)

# To capture VMT of Taxis (TRPTRANS==17), assign DRVR_FLG == 1
trip$DRVR_FLG <- ifelse(trip$TRPTRANS==17, 1, trip$DRVR_FLG)

# Create CarDriver and CarPassenger modes

trip$mode_name <- ifelse(trip$mode_name=="Car" & trip$DRVR_FLG==1, 
                         "CarDriver",    trip$mode_name)
trip$mode_name <- ifelse(trip$mode_name=="Car" & trip$PSGR_FLG==1,
                         "CarPassenger", trip$mode_name)

# Include relevant modes 
trip <- trip[trip$mode_name %in% c("Bike", "Bus", "CarDriver", "CarPassenger", 
                                   "Motorcycle", "Rail", "Walk"), ]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Aggregate Miles by person and mode
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmiles <- aggregate(TRPMILES ~ person_id+mode_name, data = trip, sum)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reshape trip miles file from long to wide
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmiles_wide <- reshape(trip_pmiles,
             timevar = "mode_name",
             idvar = c("person_id"),
             direction = "wide")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with person file to pick up non travelers and person weight
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmiles_wide <- merge(person,trip_pmiles_wide, by ="person_id", all.x = TRUE)

# assign all missing values to 0 miles traveled
trip_pmiles_wide[is.na(trip_pmiles_wide)] <- 0

# create region
trip_pmiles_wide$Region <- "United States"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Calculated weighted means, SEs, and CV of VTM/PMT
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Set up survey design 
library(survey)
survey <- svydesign(id=~HOUSEID, strata=~HHSTATE, weights=~WTPERFIN, 
                    data=trip_pmiles_wide)

flist <- c("Bike", "Walk", "CarDriver", "CarPassenger", "Bus", "Rail", "Motorcycle")
loop1 <- function( vars){ svymean( as.formula( paste0("~TRPMILES.", vars)),  
          design=survey) }
out1 <- as.data.frame(lapply( flist, loop1))

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Reformat output in to table (Per capita annual miles by mode)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# create empty data frame
baseline2015 <- data.frame(Region = rep("United States",8), 
                           Item_Name = rep("Distances (miles per year)",8),
                           Scenario_ID = rep("Baseline2015", 8),
                           Mode = c("Bike", "Bus", "CarDriver","CarPassenger",
                                    "Motorcycle", "Rail","Truck", "Walk"),
                           Item_Result = rep(NA, 8),
                           se=rep(NA, 8), stringsAsFactors=FALSE)  
baseline2015[baseline2015$Mode=="Bike",5:6]         <- out1[1,1:2]*365
baseline2015[baseline2015$Mode=="Bus",5:6]          <- out1[1,9:10]*365
baseline2015[baseline2015$Mode=="CarDriver",5:6]    <- out1[1,5:6]*365
baseline2015[baseline2015$Mode=="CarPassenger",5:6] <- out1[1,7:8]*365
baseline2015[baseline2015$Mode=="Motorcycle",5:6]   <- out1[1,13:14]*365
baseline2015[baseline2015$Mode=="Rail",5:6]         <- out1[1,11:12]*365
baseline2015[baseline2015$Mode=="Truck",5:6]        <- c(870.79,NA)
baseline2015[baseline2015$Mode=="Walk",5:6]         <- out1[1,3:4]*365

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Write out file to default narratives file

setwd("C:/Users/Noneof/Documents/HealthLinks/ITHIM_MPO/USA/CalibrationData/NHTS2017")
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(baseline2015, paste0("default_narratives",format(Sys.time(),
           "%Y-%m-%d"),".csv"), row.names=FALSE)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Aggregate trip durations by person and mode
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmin   <- aggregate(TRVLCMIN ~ person_id+mode_name, data = trip, sum)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reshape trip minutes file from long to wide
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmin_wide <- reshape(trip_pmin,
                            timevar = "mode_name",
                            idvar = c("person_id"),
                            direction = "wide")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with person file to pick up non travelers and person weight
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

trip_pmin_wide <- merge(person,trip_pmin_wide, by ="person_id", all.x = TRUE)

# assign all missing values to 0 miles traveled
trip_pmin_wide[is.na(trip_pmin_wide)] <- 0

# Subset walk and bike mode
trip_pmin_wide <- trip_pmin_wide[,c("HHSTATE", "HOUSEID", "WTPERFIN", "Age",
                  "Sex", "TRVLCMIN.Bike", "TRVLCMIN.Walk", "TRVLCMIN.Bus", 
                  "TRVLCMIN.Rail")]

# create region
trip_pmin_wide$Region <- "United States"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Calculate weighted means/p/week and SEs by age, sex and mode of 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Set up survey design 
survey <- svydesign(id=~HOUSEID, strata=~HHSTATE, weights=~WTPERFIN, 
                    data=trip_pmin_wide)

# Age-Sex stratified
flist2 <- c("Bike" , "Walk")
loop2 <- function( vars){ svyby( as.formula( paste0("~TRVLCMIN." , vars )) , 
                                 as.formula( paste("~Age+Sex")), 
                                 design=survey, svymean, vartype=c("se","cv") ) }

# Totals for Walk & Bike
loop3 <- function( vars){ svyby( as.formula( paste0("~TRVLCMIN." , vars )) , 
                                 as.formula( paste("~Region")), 
                                 design=survey, svymean, vartype=c("se","cv") ) }

out2 <- as.data.frame(lapply( flist2, loop2))


out3 <- as.data.frame(lapply( flist2, loop3))

# Totals for Bus and Rail
flist3 <- c("Bus" , "Rail")
out4 <- as.data.frame(lapply( flist3, loop3))

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Reshape Age-Sex stratified file for reformated output file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

out2_long <- reshape(out2[,c("Sex", "Age", "TRVLCMIN.Bike", "TRVLCMIN.Walk")],
             varying = c("TRVLCMIN.Bike", "TRVLCMIN.Walk"),
#             idvar = NULL,
             v.names = "Baseline",
             timevar = "Mode",
             times = c("Bike", "Walk"),
             new.row.names = 1:28,
             direction = "long")

# Add Region
out2_long$Region <- "United States"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Reshape Bike, Walk totals for reformated output file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
out3$Age <- "Total"
out3$Sex <- 0
out3_long <- reshape(out3[,c("Region", "Sex", "Age", "TRVLCMIN.Bike", 
            "TRVLCMIN.Walk")], varying = c("TRVLCMIN.Bike", "TRVLCMIN.Walk"),
#           idvar = NULL,
            v.names = "Baseline",
            timevar = "Mode",
            times = c("Bike", "Walk"),
            new.row.names = 1:2,
            direction = "long")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Combine Totals into Age-Sex stratified file
#  Express as mean min per person from daily to weekly basis
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

out3_long <- rbind(out3_long, out2_long)
out3_long$Baseline <- out3_long$Baseline * 7
out3_long <- out3_long[order(out3_long$Mode,out3_long$Sex,out3_long$Age), 
                       c("Region", "Sex", "Age", "Mode","Baseline")]
out3_long$Sex <- gsub("0","Both", out3_long$Sex)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Write out file to input file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(out3_long, paste0("ATmean_min_week_age_sex_baseline",
         format(Sys.time(), "%Y-%m-%d"),".csv"), row.names=FALSE)
         
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Age-Sex Proportion of Total Active Travel Time that is Walking
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

out2$walkpct <- out2$TRVLCMIN.Walk/(out2$TRVLCMIN.Bike + out2$TRVLCMIN.Walk)

# Add 0-4 age group
out2 <- rbind(out2[out2$Age==5,],out2)
out2[1:2,c("Age")] <- 0
out2 <- out2[order(out2$Sex,out2$Age), c("Sex","Age", "walkpct")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Write out file to file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(out2, paste0("Walk_pct_AT",
          format(Sys.time(), "%Y-%m-%d"),".csv"), row.names=FALSE)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Walk/Bike; Transit Ratios (unweighted)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# All bus and rail trips
transit <- trip[trip$mode_name == "Bus" | trip$mode_name=="Rail", 
                c("person_id","TDCASEID", "TDTRPNUM","TRVLCMIN","mode_name")]

# Trips of all modes taken by transit riders 
transit <- merge(transit,trip, by = "person_id", all.x=TRUE)

# Walk and bike trips of transit riders
transit <- transit[transit$mode_name.y %in% c("Walk", "Bike"),]

# Trips before and after bus/rail trip
transit <- transit[abs(transit$TDCASEID.x - transit$TDCASEID.y) == 1, ]

# Sum minutes of Walking and cycling before and after transit trips
walk_min <- sum(transit[transit$mode_name.y == "Walk", c("TRVLCMIN.y")])
bike_min <- sum(transit[transit$mode_name.y == "Bike", c("TRVLCMIN.y")])

# Deduplicate transit file for just transit trips
transit <- transit[!duplicated(transit[c("TDCASEID.x")]),]
tran_min <- sum(transit$TRVLCMIN.x)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Write out file 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# create empty data frame
wbtratios <- data.frame(Region = rep("United States", 1), 
                        BikeTRatio=rep(NA, 1), 
                        WalkTRatio=rep(NA, 1),
                        TransitMin_p_d=rep(NA, 1),
                        stringsAsFactors=FALSE)  

wbtratios[1,2] <- bike_min/tran_min
wbtratios[1,3] <- walk_min/tran_min
wbtratios[1,4] <- 7*(out4$TRVLCMIN.Rail + out4$TRVLCMIN.Bus)

write.csv(wbtratios, paste0("WalkBikeTransitRatios",
          format(Sys.time(), "%Y-%m-%d"),".csv"), row.names=FALSE)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  US Surgeon General's Scenario: median of 150 min/p/wk (21.42857 min/p/wk)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Computes median active travel time from mean and SD of log
# normal distribution of active travel times

# Data needed in min/week
# ---------------------------
# Baseline (s)    Scenario (s)
# ---------------------------
# Mean Total AT   Mean Total AT = 45.62255
# Mean Walk AT    Mean Walk AT = 111.573928996712 mi/y = 42.91304961 min/wk
# Mean Bike AT    Mean Bike AT = 28.1787802398196 mi/y =  2.7094981  min/wk
# CV total    -0.0015429*(Mean Total AT(b) - Mean Total AT(s)) + CV total(b)
# cv_at_b      2.87946903939232  from NHANES20115-2016

###############  Baseline #############################
# Step 1 - Calculate SD
# mean is minutes per week
mean_at_b <- 45.62255
mean_at_walk_b <- 42.91304961
mean_at_bike_b <- 3.610739
cv_at_b   <- 2.87946903939232
sd_at_b   <- mean_at_b * cv_at_b
sd_at_b

# Step 2 - take natural logarithms of mean and SD

ln_mean_at_b <- log(mean_at_b) - 0.5*log(1 + (cv_at_b)^2)
ln_mean_at_b

ln_sd_at_b <- sqrt(log(1+ cv_at_b^2))
ln_sd_at_b

# Step 3 - take normal inverse
ni_ln_mean_at_b <-qnorm(0.5, ln_mean_at_b, ln_sd_at_b)
ni_ln_mean_at_b 

# Step 4 - back transform Normal inverse

med_at_b <- exp(ni_ln_mean_at_b)
med_at_b

# Step 5 - parse out walking and cycling
med_at_walk_b <- med_at_b*(1-mean_at_bike_b/mean_at_b)
med_at_walk_b
med_at_bike_b <- med_at_b- med_at_walk_b
med_at_bike_b


###############  Scenario #############################

# Step 1 - Calculate SD
# mean is minutes per week
mean_at_s <- 2*(192.04433) 
mean_at_walk_s <- 192.04433
mean_at_bike_s <- 192.04433

cv_at_s   <- -0.0015429*( mean_at_s- mean_at_b) + cv_at_b
cv_at_s

sd_at_s   <- mean_at_s * cv_at_s
sd_at_s

# Step 2 - take natural logarithms of mean and SD
ln_mean_at_s <- log(mean_at_s) - 0.5*log(1 + (cv_at_s)^2)
ln_mean_at_s

ln_sd_at_s <- sqrt(log(1+ cv_at_s^2))
ln_sd_at_s

# Step 3 - take normal inverse
ni_ln_mean_at_s <-qnorm(0.5, ln_mean_at_s, ln_sd_at_s)
ni_ln_mean_at_s 

# Step 4 - back transform Normal inverse
med_at_s <- exp(ni_ln_mean_at_s)
med_at_s

# Step 5 - parse out walking and cycling
med_at_walk_s <- med_at_s*(1-mean_at_bike_s/mean_at_s)
med_at_walk_s
med_at_bike_s <- med_at_s- med_at_walk_s
med_at_bike_s

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Convert weekly mean time into annual mean miles: Walk 3mph, Bike 12 mph
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bike_miyr <- 52*12*192.04433/60
walk_miyr <- 52*3*192.04433/60

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create output file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

ussg <- baseline2015
ussg$Scenario_ID <- "U. S. Surgeon General Recommendation"
ussg[ussg$Mode=="Bike","Item_Result"] <- bike_miyr
ussg[ussg$Mode=="Walk","Item_Result"] <- walk_miyr

# Reduce car miles retain occupancy
car_driver_pct <- ussg[ussg$Mode=="CarDriver","Item_Result"]/
  (ussg[ussg$Mode=="CarDriver","Item_Result"] + ussg[ussg$Mode=="CarPassenger","Item_Result"])

# Difference in walk and bike miles between scenarios
miles_dif <- ussg[ussg$Mode=="Bike","Item_Result"] - baseline2015[baseline2015$Mode=="Bike","Item_Result"] +
             ussg[ussg$Mode=="Walk","Item_Result"] - baseline2015[baseline2015$Mode=="Walk","Item_Result"]  

# New CarDriver miles
ussg[ussg$Mode=="CarDriver","Item_Result"] <- baseline2015[baseline2015$Mode=="CarDriver","Item_Result"] -
  miles_dif*car_driver_pct

# New CarPassenger miles
ussg[ussg$Mode=="CarPassenger","Item_Result"] <- baseline2015[baseline2015$Mode=="CarPassenger","Item_Result"] -
  miles_dif*(1 - car_driver_pct)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out US Surgeon General Scenario file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(ussg, paste0("Distances_mi_yr_ussg",
          format(Sys.time(), "%Y-%m-%d"),".csv"), row.names=FALSE)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Short trips (1.5 mi walked and 1.5-5 cycled)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# subset car trips
trip_cars <- trip[trip$mode_name=="CarDriver" | trip$mode_name=="CarPassenger",]

# create distance categories
trip_cars$discat <- as.factor(cut(trip_cars$TRPMILES,
                     breaks=c(0, 1.5, 5, max(trip_cars$TRPMILES)),
                     include.lowest = TRUE,
                     labels=c("<1.5","1.5-5",">5")))

# table(trip_cars$discat, exclude = NULL)

# merge with person file to picl up state strata for correct SE estimate
trip_cars <- merge(trip_cars,person, by = c("HOUSEID","PERSONID"), all.x=TRUE)

# set up survey design
survey <- svydesign(id=~HOUSEID, strata=~HHSTATE, weights=~WTTRDFIN, 
                    data=trip_cars)

# Sum up car miles by distance categories
car_miles_discat <- as.data.frame(svyby(~TRPMILES, ~discat, survey, svytotal))

# Total US population
pop <- sum(person$WTPERFIN)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create output file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

strips <- baseline2015
strips$Scenario_ID <- "Replacing Short Car Trips with Active Travel"

# Add Scenario Walk miles (half of miles of trips <1.5 miles divided by population) to baseline
strips[strips$Mode=="Walk", "Item_Result"] <- (car_miles_discat[car_miles_discat$discat=="<1.5", "TRPMILES"]/2)/pop +
  strips[strips$Mode=="Walk", "Item_Result"]

# Add Scenario bike miles (half of miles of trips 1.5-5 miles divided by population) to baseline
strips[strips$Mode=="Bike", "Item_Result"] <- (car_miles_discat[car_miles_discat$discat=="1.5-5", "TRPMILES"]/2)/pop +
  strips[strips$Mode=="Bike", "Item_Result"]

# Reduce car miles by 1/2 of short car trip miles maintaining baseline occupancy
# Total car miles per person
car_ttl_miles <- (car_miles_discat[car_miles_discat$discat=="<1.5", "TRPMILES"]/2 +
                 car_miles_discat[car_miles_discat$discat=="1.5-5", "TRPMILES"]/2 +
                 car_miles_discat[car_miles_discat$discat==">5", "TRPMILES"])/pop

# CarDriver
strips[strips$Mode=="CarDriver", "Item_Result"] <- car_ttl_miles *
                     (baseline2015[baseline2015$Mode=="CarDriver","Item_Result"]/
                     (baseline2015[baseline2015$Mode=="CarDriver","Item_Result"] + 
                      baseline2015[baseline2015$Mode=="CarPassenger","Item_Result"]))
# Car Passenger
strips[strips$Mode=="CarPassenger", "Item_Result"] <- car_ttl_miles - 
                               strips[strips$Mode=="CarDriver", "Item_Result"]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out US Short Trips Scenario file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(strips, paste0("Distances_mi_yr_strip",
          format(Sys.time(), "%Y-%m-%d"),".csv"), row.names=FALSE)

# Proportion of trips by mode
addmargins(tapply(trip$WTTRDFIN, trip$mode_name, sum, na.rm=TRUE))
prop.table(tapply(trip$WTTRDFIN, trip$mode_name, sum, na.rm=TRUE))