# USPopAge_Sex2015-2050.R - N Maizlish - January 6, 2020
# This program formats US population projections by age and sex from
# US Census Bureau files at:
# at https://www.census.gov/data/datasets/2014/demo/popproj/2014-popproj.html
# in the ITHIM age-sex-cause categories in 5-year increments from 2015 to 2050
# The program also calculates base year (2015) populations and proportions

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Initialize Progrram
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# turn off scientific notation in data frames
options(scipen=999)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read Census Bureau file 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pop <- read.csv(file="np2014_d1.csv", header = TRUE, stringsAsFactors = F)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Population by Age and Sex 2015 to 2050 
# Subset file for all races, origins, ethnicity and Years
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
pop <- pop[pop$origin==0 & pop$race==0 & pop$sex!=0 & pop$year %in% 
             c(2015,2020,2025,2030,2035,2040,2045,2050), ]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reshape file from wide to long for age
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

agelist <- paste0("pop_",0:100)
  
pop_long <- reshape(pop,
             varying = agelist,
             v.names = "Population",
             timevar = "age",
             times = agelist,
             direction = "long")

# Remove stem
pop_long$age <- as.numeric(gsub("pop_", "", pop_long$age))

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reformat Age and Sex Categories for ITHIM
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

age_groups <- function(x) { 
  if    (is.na(x)) y <- "NA"
  else if (x < 5)              y <- 0
  else if (x >= 5  & x <= 14)  y <- 5
  else if (x >= 15 & x <= 29)  y <- 15
  else if (x >= 30 & x <= 44)  y <- 30
  else if (x >= 45 & x <= 59)  y <- 45
  else if (x >= 60 & x <= 69)  y <- 60
  else if (x >= 70 & x <= 79)  y <- 70
  else if (x >= 80)            y <- 80
  else y <- -99
  return(y)
}

pop_long$Age <- sapply(pop_long$age,age_groups)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Aggregate by ITHIM Age Categories
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

colnames(pop_long)[colnames(pop_long) == "sex"]  <- "Sex"
colnames(pop_long)[colnames(pop_long) == "year"] <- "Year"
pop_long <- pop_long[,c("Sex", "Age", "Year", "Population")]

pop2015_2050 <- aggregate(Population ~ Sex+Age+Year, data = pop_long, FUN="sum")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Add Geography and Region Fields
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
pop2015_2050$Geography <- "United States"
pop2015_2050$Region    <- "United States"
pop2015_2050 <- pop2015_2050[order(pop2015_2050$Year, pop2015_2050$Sex, pop2015_2050$Age),
                             c("Geography","Region", "Sex","Age","Year", "Population")] 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write as a csv file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
write.csv(pop2015_2050,paste0("age_sex_region_county2010-2050", format(Sys.time(), 
          "%Y-%m-%d"),".csv"), row.names = F)


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#                                 Base year (2015)  
#
# Subset population 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pop2015 <- pop[pop$origin==0 & pop$race==0 & pop$sex!=0 & pop$year == 2015, ]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Reshape file from wide to long for age
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pop2015_long <- reshape(pop2015,
                    varying = agelist,
                    v.names = "Population",
                    timevar = "age",
                    times = agelist,
                    direction = "long")

# Remove stem
pop2015_long$age <- as.numeric(gsub("pop_", "", pop2015_long$age))

pop2015_long$Age <- sapply(pop2015_long$age,age_groups)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Aggregate by ITHIM Age Categories
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

colnames(pop2015_long)[colnames(pop2015_long) == "sex"]  <- "Sex"
pop2015_long <- pop2015_long[,c("Sex", "Age", "Population")]

pop2015base <- aggregate(Population ~ Sex+Age, data = pop2015_long, FUN="sum")


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Calculate Percent of Total Population
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Total Population (new row)
pop2015base[17,] <- colSums(pop2015base)

pop2015base[17,c("Sex")] <- 0
pop2015base[17,c("Age")] <- "Total"

# Sort so total is first row
pop2015base <- pop2015base[order(pop2015base$Sex, pop2015base$Age), ]
pop2015base[1,c("Sex")] <- "Both"

# calculate proportion
pop2015base$Percent <-  pop2015base$Population/pop2015base[1,c("Population")]

# Add region
pop2015base$Region <- "United States"

pop2015base <- pop2015base[,c("Region", "Sex","Age", "Population", "Percent")] 

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write as a csv file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
write.csv(pop2015base,paste0("age_sex_region2010", format(Sys.time(), 
          "%Y-%m-%d"),".csv"), row.names = F)

